@@ -1,11 +1,10 @@
 ;;; erc-bbdb.el --- Integrating the BBDB into ERC
 
+;; Copyright (C) 2001,2002,2004 Free Software Foundation, Inc.
+
 ;; Author: Andreas Fuchs <asf@void.at>
 ;; Maintainer: Mario Lang <mlang@delysid.org>
 
-;; Copyright (C) 2001 Andreas Fuchs <asf@void.at>
-;; Copyright (C) 2002 by Mario Lang <mlang@delysid.org>
-
 ;; This is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
@@ -42,7 +41,7 @@
 (require 'bbdb-gui)
 (require 'bbdb-hooks)
 
-(defconst erc-bbdb-version "$Revision: 1.20 $"
+(defconst erc-bbdb-version "$Revision: 1.23.2.2 $"
   "ERC BBDB revision.")
 
 (defgroup erc-bbdb nil
@@ -139,17 +138,19 @@ or a person joined a channel visible on any frame."
 
 (defun erc-bbdb-whois (proc parsed)
   (let (; We could use server name too, probably
-	(nick (aref parsed 3))
-	(name (aref parsed 7))
-	(finger-host (concat (aref parsed 4) "@" (aref parsed 5))))
+	(nick (second (erc-response.command-args parsed)))
+	(name (erc-response.contents parsed))
+	(finger-host (concat (third (erc-response.command-args parsed))
+                             "@"
+                             (fourth (erc-response.command-args parsed)))))
     (erc-bbdb-insinuate-and-show-entry erc-bbdb-auto-create-on-whois-p proc
 				       nick name finger-host t)))
 
 (defun erc-bbdb-JOIN (proc parsed)
-  (let* ((sender (erc-parse-user (aref parsed 1)))
+  (let* ((sender (erc-parse-user (erc-response.sender parsed)))
 	 (nick (nth 0 sender)))
     (unless (string= nick (erc-current-nick))
-      (let* ((channel (aref parsed 2))
+      (let* ((channel (erc-response.contents parsed))
 	     (finger-host (concat (nth 1 sender) "@" (nth 2 sender))))
 	  (erc-bbdb-insinuate-and-show-entry
 	   erc-bbdb-auto-create-on-join-p proc
@@ -157,13 +158,13 @@ or a person joined a channel visible on any frame."
 
 (defun erc-bbdb-NICK (proc parsed)
   "Annotate new nick name to a record in case it already exists."
-  (let* ((sender (erc-parse-user (aref parsed 1)))
+  (let* ((sender (erc-parse-user (erc-response.sender parsed)))
 	 (nick (nth 0 sender)))
     (unless (string= nick (erc-current-nick))
       (let* ((finger-host (concat (nth 1 sender) "@" (nth 2 sender))))
 	(erc-bbdb-insinuate-and-show-entry
 	 erc-bbdb-auto-create-on-nick-p proc
-	 nick nil finger-host nil (aref parsed 2))))))
+	 nick nil finger-host nil (erc-response.contents parsed))))))
 
 (defun erc-bbdb-init-highlighting-hook-fun (proc parsed)
   (erc-bbdb-init-highlighting))
@@ -190,9 +191,9 @@ counterparts `erc-pals', `erc-dangerous-hosts' and `erc-fools'."
 	(let ((highlight-types (split-string (cdr highlight-field)
 					     bbdb-notes-default-separator))
 	      (nick-names (split-string (cdr nick-field)
-					(concat "[\n"
+					(concat "\\(\n\\|"
 						bbdb-notes-default-separator
-						"]"))))
+						"\\)"))))
 	  (mapcar
 	   (lambda (highlight-type)
 	     (mapcar
@@ -207,16 +208,16 @@ counterparts `erc-pals', `erc-dangerous-hosts' and `erc-fools'."
 	   highlight-types)))))
 
 ;;;###autoload (autoload 'erc-bbdb-mode "erc-bbdb")
-(define-erc-module BBDB nil
+(define-erc-module bbdb nil
   "In ERC BBDB mode, you can directly interact with your BBDB."
-  ((add-hook 'erc-server-311-hook 'erc-bbdb-whois t)
-   (add-hook 'erc-server-JOIN-hook 'erc-bbdb-JOIN t)
-   (add-hook 'erc-server-NICK-hook 'erc-bbdb-NICK t)
-   (add-hook 'erc-server-376-hook 'erc-bbdb-init-highlighting-hook-fun t))
-  ((remove-hook 'erc-server-311-hook 'erc-bbdb-whois)
-   (remove-hook 'erc-server-JOIN-hook 'erc-bbdb-JOIN)
-   (remove-hook 'erc-server-NICK-hook 'erc-bbdb-NICK)
-   (remove-hook 'erc-server-376-hook 'erc-bbdb-init-highlighting-hook-fun)))
+  ((add-hook 'erc-server-311-functions 'erc-bbdb-whois t)
+   (add-hook 'erc-server-JOIN-functions 'erc-bbdb-JOIN t)
+   (add-hook 'erc-server-NICK-functions 'erc-bbdb-NICK t)
+   (add-hook 'erc-server-376-functions 'erc-bbdb-init-highlighting-hook-fun t))
+  ((remove-hook 'erc-server-311-functions 'erc-bbdb-whois)
+   (remove-hook 'erc-server-JOIN-functions 'erc-bbdb-JOIN)
+   (remove-hook 'erc-server-NICK-functions 'erc-bbdb-NICK)
+   (remove-hook 'erc-server-376-functions 'erc-bbdb-init-highlighting-hook-fun)))
 
 (provide 'erc-bbdb)
 
