@@ -1,6 +1,7 @@
 ;;; erc-match.el --- Highlight messages matching certain regexps
 
-;; Copyright (C) 2002  Andreas Fuchs <asf@void.at>
+;; Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
+;; Copyright (C) 2004 Jeremy Bertram Maitin-Shepard
 
 ;; Author: Andreas Fuchs <asf@void.at>
 ;; Keywords: comm, faces
@@ -37,26 +38,25 @@
 
 ;; Customisation:
 
-(defconst erc-match-version "$Revision: 1.15 $"
+(defconst erc-match-version "$Revision: 1.38.2.3 $"
   "ERC match mode revision.")
 
 (defgroup erc-match nil
   "Keyword and Friend/Foe/... recognition.
 Group containing all things concerning pattern matching in ERC
-messages.")
+messages."
+  :group 'erc)
 
 ;;;###autoload (autoload 'erc-match-mode "erc-match")
 (define-erc-module match nil
-  "This mode checks wether messages match certain patterns.  If so,
+  "This mode checks whether messages match certain patterns.  If so,
 they are hidden or highlighted.  This is controlled via the variables
 `erc-pals', `erc-fools', `erc-keywords', `erc-dangerous-hosts', and
-`erc-current-nick-highlight-type'.  For all these hilighting types,
-you can decide wether the entire message or only the sending nick is
-hilighted."
-  ((add-hook 'erc-insert-modify-hook 'erc-match-message)
-   (add-hook 'erc-send-modify-hook 'erc-match-message))
-  ((remove-hook 'erc-insert-modify-hook 'erc-match-message)
-   (remove-hook 'erc-send-modify-hook 'erc-match-message)))
+`erc-current-nick-highlight-type'.  For all these highlighting types,
+you can decide whether the entire message or only the sending nick is
+highlighted."
+  ((add-hook 'erc-insert-modify-hook 'erc-match-message))
+  ((remove-hook 'erc-insert-modify-hook 'erc-match-message)))
 
 ;; Remaining customizations
 
@@ -71,9 +71,13 @@ hilighted."
   :type '(repeat regexp))
 
 (defcustom erc-keywords nil
-  "List of keywords to highlight in all incoming messages."
+  "List of keywords to highlight in all incoming messages.
+Each entry in the list is either a regexp, or a cons cell with the
+regexp in the car and the face to use in the cdr.  If no face is
+specified, `erc-keyword-face' is used."
   :group 'erc-match
-  :type '(repeat regexp))
+  :type '(repeat (choice regexp
+			 (list regexp face))))
 
 (defcustom erc-dangerous-hosts nil
   "List of regexps for hosts to highlight.
@@ -82,20 +86,26 @@ Useful to mark nicks from dangerous hosts."
   :type '(repeat regexp))
 
 (defcustom erc-current-nick-highlight-type nil
-  "*Determines how to highlight your current nickname.
+  "*Determines how to highlight text in which your current nickname appears
+\(does not apply to text sent by you\).
 
 The following values are allowed:
 
-    'nick - highlight the current nickname only
-    'all  - highlight the entire message where current nickname occurs
+ nil              - do not highlight the message at all
+ 'keyword         - highlight all instances of current nickname in message
+ 'nick            - highlight the nick of the user who typed your nickname
+ 'nick-or-keyword - highlight the nick of the user who typed your nickname,
+                    or all instances of the current nickname if there was
+                    no sending user
+ 'all             - highlight the entire message where current nickname occurs
 
 Any other value disables highlighting of current nickname altogether."
   :group 'erc-match
-  :type '(choice (const :tag "highlight current nickname only" nickname)
-		 (const :tag "highlight entire message with current nickname"
-			all)
-		 (const :tag "don't highlight messages with current nickname"
-			nil)))
+  :type '(choice (const nil)
+		 (const nick)
+		 (const keyword)
+		 (const nick-or-keyword)
+		 (const all)))
 
 (defcustom erc-pal-highlight-type 'nick
   "*Determines how to highlight messages by pals.
@@ -103,14 +113,15 @@ See `erc-pals'.
 
 The following values are allowed:
 
+    nil   - do not highlight the message at all
     'nick - highlight pal's nickname only
     'all  - highlight the entire message from pal
 
 Any other value disables pal highlighting altogether."
   :group 'erc-match
-  :type '(choice (const :tag "highlight pal's nickname only" nick)
-		 (const :tag "highlight pal's entire message" all)
-		 (const :tag "don't highlight pal's messages" nil)))
+  :type '(choice (const nil)
+		 (const nick)
+		 (const all)))
 
 (defcustom erc-fool-highlight-type 'nick
   "*Determines how to highlight messages by fools.
@@ -118,14 +129,15 @@ See `erc-fools'.
 
 The following values are allowed:
 
+    nil   - do not highlight the message at all
     'nick - highlight fool's nickname only
     'all  - highlight the entire message from fool
 
 Any other value disables fool highlighting altogether."
   :group 'erc-match
-  :type '(choice (const :tag "highlight fool's nickname only" nick)
-		 (const :tag "highlight fool's entire message" all)
-		 (const :tag "don't highlight fool's messages" nil)))
+  :type '(choice (const nil)
+		 (const nick)
+		 (const all)))
 
 (defcustom erc-keyword-highlight-type 'keyword
   "*Determines how to highlight messages containing keywords.
@@ -138,9 +150,9 @@ The following values are allowed:
 
 Any other value disables keyword highlighting altogether."
   :group 'erc-match
-  :type '(choice (const :tag "highlight keywords only" keyword)
-		 (const :tag "highlight entire message containing keywords" all)
-		 (const :tag "don't highlight messages containing keywords" nil)))
+  :type '(choice (const nil)
+		 (const keyword)
+		 (const all)))
 
 (defcustom erc-dangerous-host-highlight-type 'nick
   "*Determines how to highlight messages by nicks from dangerous-hosts.
@@ -153,9 +165,9 @@ The following values are allowed:
 
 Any other value disables dangerous-host highlighting altogether."
   :group 'erc-match
-  :type '(choice (const :tag "highlight nick from dangerous-host only" nick)
-		 (const :tag "highlight entire message from dangerous-host" all)
-		 (const :tag "don't highlight messages from dangerous-hosts" nil)))
+  :type '(choice (const nil)
+		 (const nick)
+		 (const all)))
 
 
 (defcustom erc-log-matches-types-alist '((keyword . "ERC Keywords"))
@@ -165,15 +177,17 @@ Valid match type keys are:
 - pal
 - dangerous-host
 - fool
+- current-nick
 
-The other element of each cons pair in this lits is the buffer name to
+The other element of each cons pair in this list is the buffer name to
 use for the logged message."
   :group 'erc-match
   :type '(repeat (cons (choice :tag "Key"
 			       (const keyword)
 			       (const pal)
 			       (const dangerous-host)
-			       (const fool))
+			       (const fool)
+			       (const current-nick))
 		       (string :tag "Buffer name"))))
 
 (defcustom erc-log-matches-flag 'away
@@ -182,8 +196,8 @@ When nil, don't log any matched messages.
 When t, log messages.
 When 'away, log messages only when away."
   :group 'erc-match
-  :type '(choice (const away)
-		 (const nil)
+  :type '(choice (const nil)
+		 (const away)
 		 (const t)))
 
 (defcustom erc-log-match-format "%t<%n:%c> %m"
@@ -202,11 +216,35 @@ be formatted. The various format specs are:
 (defcustom erc-text-matched-hook '(erc-log-matches erc-hide-fools)
   "Hook run when text matches a given match-type.
 Functions in this hook are passed as arguments:
-(match-type nick!user@host message) where MATCH-TYPE is a symbol of:
+\(match-type nick!user@host message) where MATCH-TYPE is a symbol of:
 keyword, pal, dangerous-host, fool"
   :group 'erc-match
   :type 'hook)
 
+;; Internal variables:
+
+;; This is exactly the same as erc-button-syntax-table.  Should we
+;; just put it in erc.el
+(defvar erc-match-syntax-table
+  (let ((table (make-syntax-table)))
+    (modify-syntax-entry ?\( "w" table)
+    (modify-syntax-entry ?\) "w" table)
+    (modify-syntax-entry ?\[ "w" table)
+    (modify-syntax-entry ?\] "w" table)
+    (modify-syntax-entry ?\{ "w" table)
+    (modify-syntax-entry ?\} "w" table)
+    (modify-syntax-entry ?` "w" table)
+    (modify-syntax-entry ?' "w" table)
+    (modify-syntax-entry ?^ "w" table)
+    (modify-syntax-entry ?- "w" table)
+    (modify-syntax-entry ?_ "w" table)
+    (modify-syntax-entry ?| "w" table)
+    (modify-syntax-entry ?\\ "w" table)
+    table)
+  "Syntax table used when highlighting messages.
+This syntax table should make all the legal nick characters word
+constituents.")
+
 ;; Faces:
 
 (defface erc-current-nick-face '((t (:bold t :foreground "DarkTurquoise")))
@@ -230,7 +268,8 @@ See `erc-fools'."
 
 (defface erc-keyword-face '((t (:bold t :foreground "pale green")))
   "ERC face for your keywords.
-See `erc-keywords'."
+Note that this is the default face to use if
+`erc-keywords' does not specify another."
   :group 'erc-faces)
 
 ;; Functions:
@@ -239,35 +278,43 @@ See `erc-keywords'."
   "Add an entry interactively to a list.
 LIST must be passed as a symbol
 The query happens using PROMPT.
-Completion is performed on the optional alist COMPLETIONS.
-Before adding the new entry, it is tested using TEST.
-TEST must be a function accepting a regular expression."
+Completion is performed on the optional alist COMPLETIONS."
   (let ((entry (completing-read
 		prompt
 		completions
 		(lambda (x)
-		  (not (member (car x) (eval list)))))))
-    (if (member entry (eval list))
+		  (not (erc-member-ignore-case (car x) (symbol-value list)))))))
+    (if (erc-member-ignore-case entry (symbol-value list))
 	(error (format "\"%s\" is already on the list" entry))
       (set list (cons entry (symbol-value list))))))
 
 (defun erc-remove-entry-from-list (list prompt)
   "Remove an entry interactively from a list.
-LIST must be passed as a symbol
-The query happens using PROMPT and predicate TEST."
-  (let ((entry (completing-read
-		prompt
-		(mapcar 'list (symbol-value list))
-		(lambda (x)
-		  (member (car x) (eval list)))
-		t)))
-    (set list (delete entry (symbol-value list)))))
+LIST must be passed as a symbol.
+The elements of LIST can be strings, or cons cells where the
+car is the string."
+  (let* ((alist (mapcar (lambda (x)
+			  (if (listp x)
+			      x
+			    (list x)))
+			(symbol-value list)))
+	 (entry (completing-read
+		 prompt
+		 alist
+		 nil
+		 t)))
+    (if (erc-member-ignore-case entry (symbol-value list))
+	;; plain string
+	(set list (delete entry (symbol-value list)))
+      ;; cons cell
+      (set list (delete (assoc entry (symbol-value list))
+			(symbol-value list))))))
 
 ;;;###autoload
 (defun erc-add-pal ()
   "Add pal interactively to `erc-pals'."
   (interactive)
-  (erc-add-entry-to-list 'erc-pals "Add pal: " channel-members))
+  (erc-add-entry-to-list 'erc-pals "Add pal: " (erc-get-server-nickname-alist)))
 
 ;;;###autoload
 (defun erc-delete-pal ()
@@ -279,7 +326,7 @@ The query happens using PROMPT and predicate TEST."
 (defun erc-add-fool ()
   "Add fool interactively to `erc-fools'."
   (interactive)
-  (erc-add-entry-to-list 'erc-fools "Add fool: " channel-members))
+  (erc-add-entry-to-list 'erc-fools "Add fool: " (erc-get-server-nickname-alist)))
 
 ;;;###autoload
 (defun erc-delete-fool ()
@@ -311,45 +358,55 @@ The query happens using PROMPT and predicate TEST."
   (interactive)
   (erc-remove-entry-from-list 'erc-dangerous-hosts "Delete dangerous-host: "))
 
-(defun erc-current-nick-p (nickuserhost msg)
+(defun erc-match-current-nick-p (nickuserhost msg)
   "Check whether the current nickname is in MSG.
 NICKUSERHOST will be ignored."
-  (and msg
-       (string-match (erc-current-nick) msg)))
+  (with-syntax-table erc-match-syntax-table
+    (and msg
+         (string-match (concat "\\b"
+			       (regexp-quote (erc-current-nick))
+			       "\\b")
+		       msg))))
 
-(defun erc-pal-p (nickuserhost msg)
+(defun erc-match-pal-p (nickuserhost msg)
   "Check whether NICKUSERHOST is in `erc-pals'.
 MSG will be ignored."
   (and nickuserhost
        (erc-list-match erc-pals nickuserhost)))
 
-(defun erc-fool-p (nickuserhost msg)
+(defun erc-match-fool-p (nickuserhost msg)
   "Check whether NICKUSERHOST is in `erc-fools' or MSG is directed at a fool."
   (and msg nickuserhost
        (or (erc-list-match erc-fools nickuserhost)
-	   (erc-directed-at-fool-p msg))))
+	   (erc-match-directed-at-fool-p msg))))
 
-(defun erc-keyword-p (nickuserhost msg)
+(defun erc-match-keyword-p (nickuserhost msg)
   "Check whether any keyword of `erc-keywords' matches for MSG.
 NICKUSERHOST will be ignored."
   (and msg
-       (erc-list-match erc-keywords msg)))
+       (erc-list-match
+	(mapcar (lambda (x)
+		  (if (listp x)
+		      (car x)
+		    x))
+		erc-keywords)
+	msg)))
 
-(defun erc-dangerous-host-p (nickuserhost msg)
+(defun erc-match-dangerous-host-p (nickuserhost msg)
   "Check whether NICKUSERHOST is in `erc-dangerous-hosts'.
 MSG will be ignored."
   (and nickuserhost
        (erc-list-match erc-dangerous-hosts nickuserhost)))
 
-(defun erc-directed-at-fool-p (msg)
-  "Check wether MSG is directed at a fool.
+(defun erc-match-directed-at-fool-p (msg)
+  "Check whether MSG is directed at a fool.
 In order to do this, every entry in `erc-fools' will be used.
 In any of the following situations, MSG is directed at an entry FOOL:
 
-- MSG starts with \"FOOL: \"
+- MSG starts with \"FOOL: \" or \"FOO, \"
 - MSG contains \", FOOL.\" (actually, \"\\s. FOOL\\s.\")"
   (let ((fools-beg (mapcar (lambda (entry)
-				 (concat "^" entry ": "))
+				 (concat "^" entry "[:,] "))
 			   erc-fools))
 	(fools-end (mapcar (lambda (entry)
 				 (concat "\\s. " entry "\\s."))
@@ -363,8 +420,7 @@ In any of the following situations, MSG is directed at an entry FOOL:
 
 (defun erc-get-parsed-vector-nick (vect)
   "Return nickname in the parsed vector VECT."
-  (let* ((untreated-nick (when (vectorp vect)
-			   (aref vect 1)))
+  (let* ((untreated-nick (and vect (erc-response.sender vect)))
 	 (maybe-nick (when untreated-nick
 		       (car (split-string untreated-nick "!")))))
     (when (and (not (null maybe-nick))
@@ -374,11 +430,12 @@ In any of the following situations, MSG is directed at an entry FOOL:
 (defun erc-get-parsed-vector-type (vect)
   "Return message type in the parsed vector VECT."
   (and vect
-       (aref vect 0)))
+       (erc-response.command vect)))
 
 (defun erc-match-message ()
   "Mark certain keywords in a region.
 Use this defun with `erc-insert-modify-hook'."
+  ;; This needs some refactoring.
   (goto-char (point-min))
   (let* ((to-match-nick-dep '("pal" "fool" "dangerous-host"))
 	 (to-match-nick-indep '("keyword" "current-nick"))
@@ -403,41 +460,66 @@ Use this defun with `erc-insert-modify-hook'."
        (lambda (match-type)
 	 (goto-char (point-min))
 	 (let* ((match-prefix (concat "erc-" match-type))
-		(match-pred (intern (concat match-prefix "-p")))
+		(match-pred (intern (concat "erc-match-" match-type "-p")))
 		(match-htype (eval (intern (concat match-prefix
 						   "-highlight-type"))))
 		(match-regex (if (string= match-type "current-nick")
-				 (erc-current-nick)
+				 (regexp-quote (erc-current-nick))
 			       (eval (intern (concat match-prefix "s")))))
 		(match-face (intern (concat match-prefix "-face"))))
 	   (when (funcall match-pred nickuserhost message)
 	     (cond
+              ;; Highlight the nick of the message
 	      ((and (eq match-htype 'nick)
-		    nick-end)
+                    nick-end)
 	       (erc-put-text-property
 		nick-beg nick-end
 		'face match-face (current-buffer)))
+              ;; Highlight the nick of the message, or the current
+              ;; nick if there's no nick in the message (e.g. /NAMES
+              ;; output)
+              ((and (string= match-type "current-nick")
+                    (eq match-htype 'nick-or-keyword))
+               (if nick-end 
+                   (erc-put-text-property
+                    nick-beg nick-end
+                    'face match-face (current-buffer))
+                 (goto-char (+ 2 (or nick-end
+                                     (point-min))))
+                 (while (re-search-forward match-regex nil t)
+                   (erc-put-text-property (match-beginning 0) (match-end 0)
+                                          'face match-face))))
+              ;; Highlight the whole message
 	      ((eq match-htype 'all)
 	       (erc-put-text-property
 		(point-min) (point-max)
 		'face match-face (current-buffer)))
+              ;; Highlight all occurrences of the word to be
+              ;; highlighted.
 	      ((and (string= match-type "keyword")
-		    (eq match-htype 'keyword))
-	       (mapc (lambda (regex)
-		       (goto-char (+ 2 (or nick-end
-					   (point-min))))
-		       (while (re-search-forward regex nil t)
-			 (erc-put-text-property
-			  (match-beginning 0) (match-end 0)
-			  'face match-face)))
+                    (eq match-htype 'keyword))
+	       (mapc (lambda (elt)
+		       (let ((regex elt)
+			     (face match-face))
+			 (when (consp regex)
+			   (setq regex (car elt)
+				 face (cdr elt)))
+			 (goto-char (+ 2 (or nick-end
+					     (point-min))))
+			 (while (re-search-forward regex nil t)
+			   (erc-put-text-property
+			    (match-beginning 0) (match-end 0)
+			    'face face))))
 		     match-regex))
+              ;; Highlight all occurrences of our nick.
 	      ((and (string= match-type "current-nick")
-		    (eq match-htype 'nick))
+		    (eq match-htype 'keyword))
 	       (goto-char (+ 2 (or nick-end
 				   (point-min))))
 	       (while (re-search-forward match-regex nil t)
 		 (erc-put-text-property (match-beginning 0) (match-end 0)
 					'face match-face)))
+              ;; Else twiddle your thumbs.
 	      (t nil))
 	     (run-hook-with-args
 	      'erc-text-matched-hook
@@ -522,7 +604,7 @@ deactivate/activate match logging in the latter. See
   nil)
 
 ; This handler must be run _before_ erc-process-away is.
-(add-hook 'erc-server-305-hook 'erc-log-matches-come-back nil)
+(add-hook 'erc-server-305-functions 'erc-log-matches-come-back nil)
 
 (defun erc-go-to-log-matches-buffer ()
   "Interactively open an erc-log-matches buffer."
@@ -539,7 +621,7 @@ deactivate/activate match logging in the latter. See
 
 (defun erc-hide-fools (match-type nickuserhost message)
  "Hide foolish comments.
-This function should be called from `erc-text-matches-hook'."
+This function should be called from `erc-text-matched-hook'."
  (when (eq match-type 'fool)
    (erc-put-text-properties (point-min) (point-max)
 			    '(invisible intangible)
@@ -548,3 +630,9 @@ This function should be called from `erc-text-matches-hook'."
 (provide 'erc-match)
 
 ;;; erc-match.el ends here
+;;
+;; Local Variables:
+;; indent-tabs-mode: t
+;; tab-width: 8
+;; standard-indent: 4
+;; End:
