@@ -1,5 +1,41 @@
+;;; erc-nets.el --- IRC networks
+
+;; Copyright (C) 2002,2004 Free Software Foundation, Inc.
+
+;; Author: Mario Lang <mlang@lexx.delysid.org>
+;; Keywords: comm
+
+;;; Commentary:
+
+;; This file deals with IRC networks.
+;;
+;; Usage:
+;;
+;; Put into your .emacs:
+;;
+;; (require 'erc-nets)
+;;
+;; M-x erc-server-select provides an alternative way to connect to servers by
+;; choosing networks.
+;; You can use (eq (erc-network) 'Network) if you'd like to set variables or do
+;; certain actions according to which network you're connected to.
+;; If a network you use is not listed in `erc-networks-alist', you can put
+;; (add-to-list 'erc-networks-alist '(Network "irc.server-name.net")) in your
+;; config file.
+
+;;; Code:
+
 (require 'erc)
 
+(defconst erc-nets-version "$Revision: 1.17.2.2 $"
+  "ERC networks revision.")
+
+;; Variables
+
+(defgroup erc-networks nil
+  "IRC Networks"
+  :group 'erc)
+
 (defcustom erc-server-alist
 '(("4-irc: Random server" 4-irc "4-irc.com" 6667)
   ("A5KNet: Random server" A5KNet "irc.a5knet.com" ((6660 6669)))
@@ -29,8 +65,8 @@
   ("Blitzed: Random server" Blitzed "irc.blitzed.org" (6667 7000 ))
   ("Brasirc: Random server" Brasirc "irc.brasirc.net" ((6666 6667)))
   ("Brasirc: BR, PA, Belem" Brasirc "irc.libnet.com.br" ((6666 6668) 7777 8002 ))
-  ("Brasnet: Random European server" Brasnet "eu.brasnet.org" ((6665 6669)))
-  ("Brasnet: Random US server" Brasnet "us.brasnet.org" ((6665 6669)))
+  ("BRASnet: Random European server" BRASnet "eu.brasnet.org" ((6665 6669)))
+  ("BRASnet: Random US server" BRASnet "us.brasnet.org" ((6665 6669)))
   ("BubbleNet: Random server" BubbleNet "irc.bubblenet.org" ((6667 6669)))
   ("CCnet: Random server" CCnet "irc.cchat.net" (6667 7000 ))
   ("CCnet: US, TX, Dallas" CCnet "irc2.cchat.net" (6667 7000 ))
@@ -141,6 +177,9 @@
   ("Fraggers: Random server" Fraggers "irc.fraggers.co.uk" ((6661 6669) (7000 7001) ))
   ("FreedomChat: Random server" FreedomChat "chat.freedomchat.net" 6667)
   ("FreedomIRC: Random server" FreedomIRC "irc.freedomirc.net" 6667)
+  ("Freenode: Random server" freenode "irc.freenode.net" 6667)
+  ("Freenode: Random EU server" freenode "irc.eu.freenode.net" 6667)
+  ("Freenode: Random US server" freenode "irc.us.freenode.net" 6667)
   ("FunNet: Random server" FunNet "irc.funnet.org" 6667)
   ("Galaxynet: Random server" GalaxyNet "irc.galaxynet.org" ((6662 6668) 7000 ))
   ("Galaxynet: AU, NZ, Auckland" GalaxyNet "auckland.nz.galaxynet.org" ((6661 6669)))
@@ -262,9 +301,6 @@
   ("Nullrouted: Random server" Nullrouted "irc.nullrouted.org" ((6666 6669) 7000 ))
   ("NullusNet: Random server" NullusNet "irc.nullus.net" 6667)
   ("OpChat: Random server" OpChat "irc.opchat.org" ((6667 6669)))
-  ("Openprojects: Random server" Openprojects "irc.openprojects.net" 6667)
-  ("Openprojects: Random EU server" Openprojects "irc.eu.openprojects.net" 6667)
-  ("Openprojects: Random US server" Openprojects "irc.us.openprojects.net" 6667)
   ("Othernet: Random server" Othernet "irc.othernet.org" 6667)
   ("Othernet: US, FL, Miami" Othernet "miami.fl.us.othernet.org" 6667)
   ("Othernet: US, MO, StLouis" Othernet "stlouis.mo.us.othernet.org" 6667)
@@ -390,290 +426,371 @@ NET is a symbol indicating to which network from `erc-networks-alist' this
   server corresponds,
 HOST is the servers hostname and
 PORTS is either a number, a list of numbers, or a list of port ranges."
-  :group 'erc
+  :group 'erc-networks
   :type 'sexp)
 
 (defcustom erc-networks-alist
-'((4-irc nil)
-  (A5KNet nil)
-  (AbleNet nil)
-  (Accessirc nil)
-  (Acestar nil)
-  (Action-IRC nil)
-  (AfterNET nil)
-  (Alternativenet nil)
-  (AmigaNet nil)
-  (AngelEyez nil)
-  (Anothernet nil)
-  (ArabChat nil)
-  (AsiaTalk nil)
-  (AstroLink nil)
-  (Asylumnet nil)
-  (Austnet nil)
-  (AwesomeChat nil)
-  (Awesomechristians nil)
-  (Axenet nil)
-  (BGIRC nil)
-  (Beyondirc nil)
-  (Blabbernet nil)
-  (Blitzed nil)
-  (Brasirc nil)
-  (Brasnet nil)
-  (BubbleNet nil)
-  (CCnet nil)
-  (CNN nil)
-  (Chat-Net nil)
-  (Chat-Solutions nil)
-  (ChatCircuit nil)
-  (ChatNut nil)
-  (ChatPR nil)
-  (ChatX nil)
-  (Chatcafe nil)
-  (Chatchannel nil)
-  (Chatlink nil)
-  (Chatnet nil)
-  (Chatpinoy nil)
-  (Chatroom nil)
-  (Chatster nil)
-  (China263 nil)
-  (Cineplex1 nil)
-  (CobraNet nil)
-  (Coolchat nil)
-  (Criten nil)
-  (CyGanet nil)
-  (Cyberchat nil)
-  (DALnet nil)
-  (DWChat nil)
-  (Dark-Tou-Net nil)
-  (DarkMyst nil)
-  (Darkfire nil)
-  (Darkserv nil)
-  (Darksystem nil)
-  (Darktree nil)
-  (DayNet nil)
-  (Deepspace nil)
-  (Different nil)
-  (Digarix nil)
-  (Digatech nil)
-  (Digital-Base nil)
-  (Digitalirc nil)
-  (Discussioni nil)
-  (DorukNet nil)
-  (Dreamcast nil)
-  (Dynastynet nil)
-  (EFnet nil)
-  (EgyptianIRC nil)
-  (Eircnet nil)
-  (Eleethal nil)
-  (EntertheGame nil)
-  (EpiKnet nil)
-  (EsperNet nil)
-  (Esprit nil)
-  (Evilzinc nil)
-  (ExodusIRC nil)
-  (FDFnet nil)
-  (FEFnet nil)
-  (Financialchat nil)
-  (Forestnet nil)
-  (ForeverChat nil)
-  (Fraggers nil)
-  (FreedomChat nil)
-  (FreedomIRC nil)
-  (FunNet nil)
-  (GIKInet nil)
-  (GRnet nil)
-  (GalaxyNet nil)
-  (Gamesnet nil)
-  (GammaForce nil)
-  (GizNet nil)
-  (GlobIRC nil)
-  (Globalchat nil)
-  (Goldchat nil)
-  (Goodchatting nil)
-  (GravityLords nil)
-  (GulfChat nil)
-  (HabberNet nil)
-  (HanIRC nil)
-  (Hellenicnet nil)
-  (ICQnet nil)
-  (IRC-Chile nil)
-  (IRC-Hispano nil)
-  (IRCGate nil)
-  (IRCGeeks nil)
-  (IRCSoulZ nil)
-  (IRCSul nil)
-  (IRCchat nil)
-  (IRChat nil)
-  (IRCnet nil)
-  (IRCtown nil)
-  (IRCworld nil)
-  (IceNet nil)
-  (Infatech nil)
-  (Infinity nil)
-  (Infomatrix nil)
-  (Inside3D nil)
-  (InterlinkChat nil)
-  (IrcLordz nil)
-  (IrcMalta nil)
-  (IrcTalk nil)
-  (Irctoo nil)
-  (Israelnet nil)
-  (K0wNet nil)
-  (KDFSnet nil)
-  (Kemik nil)
-  (Kewl.Org nil)
-  (Kickchat nil)
-  (KidsWorld nil)
-  (Knightnet nil)
-  (Konfido.Net nil)
-  (Kreynet nil)
-  (Krono nil)
-  (Krushnet nil)
-  (LagNet nil)
-  (Librenet nil)
-  (LinkNet nil)
-  (Liquidized nil)
-  (M-IRC nil)
-  (MagicStar nil)
-  (Mavra nil)
-  (MediaDriven nil)
-  (Morat nil)
-  (MusicCity nil)
-  (Mysteria nil)
-  (Mysterychat nil)
-  (Mystical nil)
-  (Narancs nil)
-  (Net-France nil)
-  (Nevernet nil)
-  (Newnet nil)
-  (Nexusirc nil)
-  (NightStar nil)
-  (NitrousNet nil)
-  (Novernet nil)
-  (Nullrouted nil)
-  (NullusNet nil)
-  (OpChat nil)
-  (Openprojects
-   "openprojects.net$")
-  (OtherSide nil)
-  (Othernet nil)
-  (Outsiderz nil)
-  (OzOrg nil)
-  (POLNet nil)
-  (PTirc nil)
-  (PTlink nil)
-  (PTnet nil)
-  (Peacefulhaven nil)
-  (PhazedIRC nil)
-  (Philchat nil)
-  (PiNet nil)
-  (Pinoycentral nil)
-  (Planetarion nil)
-  (Psionics nil)
-  (QChat nil)
-  (QuakeNet nil)
-  (Realirc nil)
-  (RealmNET nil)
-  (Rebelchat nil)
-  (Red-Latina nil)
-  (RedLatona nil)
-  (Relicnet nil)
-  (Rezosup nil)
-  (Risanet nil)
-  (Rubiks nil)
-  (Rusnet nil)
-  (Sandnet nil)
-  (Scunc nil)
-  (SerbianCafe nil)
-  (SexNet nil)
-  (ShadowFire nil)
-  (ShadowWorld nil)
-  (SkyNet nil)
-  (Slashnet nil)
-  (SolarStone nil)
-  (Sorcery nil)
-  (SourceIRC nil)
-  (SpaceTronix nil)
-  (Spirit-Harmony nil)
-  (StarChat nil)
-  (StarEquinox nil)
-  (StarWars-IRC nil)
-  (Starlink nil)
-  (Stormdancing nil)
-  (Superchat nil)
-  (Sysopnet nil)
-  (TR-net nil)
-  (Telstra nil)
-  (Tri-net nil)
-  (TriLink nil)
-  (TurkishChat nil)
-  (UICN nil)
-  (UberNinja nil)
-  (UltraIRC nil)
-  (UnderChat nil)
-  (UnderZ nil)
-  (Undernet nil)
-  (UniChat nil)
-  (UnionLatina nil)
-  (Univers nil)
-  (UnixR nil)
-  (Vidgamechat nil)
-  (VirtuaNet nil)
-  (Vitamina nil)
-  (Voila nil)
-  (Wahou nil)
-  (Warpednet nil)
-  (Weaklinks nil)
-  (Webnet nil)
-  (WinChat nil)
-  (WinIRC nil)
-  (WorldIRC nil)
-  (WyldRyde nil)
-  (XNet nil)
-  (XWorld nil)
-  (XentoniX nil)
-  (Xevion nil)
-  (ZAnetNet nil)
-  (ZAnetOrg nil)
-  (ZUHnet nil)
-  (ZiRC nil)
-  (Zurna nil)
-  (euIRC nil)
-  (ircXtreme nil)
-  (mIRC-X nil)
-  (phrozN nil)
-  (starlink-irc nil))
+  '((4-irc "4-irc.com")
+    (A5KNet "a5knet.com")
+    (AbleNet "ablenet.org")
+    (Accessirc "accessirc.net")
+    (Acestar "acestar.org")
+    (Action-IRC "action-irc.net")
+    (AfterNET "afternet.org")
+    (Alternativenet "altnet.org")
+    (AmigaNet "amiganet.org")
+    (AngelEyez "angeleyez.net")
+    (Anothernet "another.net")
+    (ArabChat "arabchat.org")
+    (AsiaTalk "asiatalk.org")
+    (AstroLink "astrolink.org")
+    (Asylumnet "asylumnet.org")
+    (Austnet "austnet.org")
+    (AwesomeChat "awesomechat.net")
+    (Awesomechristians "awesomechristians.com")
+    (Axenet "axenet.org")
+    (Beyondirc "beyondirc.net")
+    (BGIRC "bulgaria.org")
+    (Blabbernet "blabber.net")
+    (Blitzed "blitzed.org")
+    (BrasIRC "brasirc.net")
+    (BRASnet "brasnet.org")
+    (BubbleNet "bubblenet.org")
+    (CCnet "christian-chat.net")
+    (Chat-Net "chat-net.org")
+    (Chat-Solutions "chat-solutions.org")
+    (Chatcafe "chatcafe.net")
+    (Chatchannel "chatchannel.org")
+    (ChatCircuit "chatcircuit.com")
+    (Chatlink "chatlink.org")
+    (Chatnet "chatnet.org")
+    (ChatNut "chatnut.net")
+    (Chatpinoy "chatpinoy.com")
+    (ChatPR "chatpr.org")
+    (Chatroom "chatroom.org")
+    (Chatster "chatster.org")
+    (ChatX "chatx.net")
+    (China263 "263.net")
+    (Cineplex1 "cineplex1.com")
+    (CNN "cnn.com")
+    (CobraNet "cobra.net")
+    (Coolchat "coolchat.net")
+    (Criten "criten.net")
+    (Cyberchat "cyberchat.org")
+    (CyGanet "cyga.net")
+    (DALnet "dal.net")
+    (Dark-Tou-Net "d-t-net.de")
+    (Darkfire "darkfire.net")
+    (DarkMyst "darkmyst.org")
+    (Darkserv "darkserv.net")
+    (Darksystem "darksystem.com")
+    (Darktree "darktree.net")
+    (DayNet "daynet.org")
+    (Deepspace "deepspace.org")
+    (Different "different.net")
+    (Digarix "digarix.net")
+    (Digatech "digatech.net")
+    (Digital-Base "digital-base.net")
+    (Digitalirc "digitalirc.net")
+    (Discussioni "discussioni.org")
+    (DorukNet "doruk.net.tr")
+    (DWChat "dwchat.net")
+    (Dynastynet "dynastynet.net")
+    (EFnet nil)
+    (EgyptianIRC "egyptianirc.net")
+    (Eircnet "eircnet.org")
+    (Eleethal "eleethal.com")
+    (EntertheGame "enterthegame.com")
+    (EpiKnet "epiknet.org")
+    (EsperNet "esper.net")
+    (Esprit "esprit.net")
+    (euIRC "euirc.net")
+    (Evilzinc "evilzinc.net")
+    (ExodusIRC "exodusirc.net")
+    (FDFnet "fdfnet.net")
+    (FEFnet "fef.net")
+    (Financialchat "financialchat.com")
+    (Forestnet "forestnet.org")
+    (ForeverChat "foreverchat.net")
+    (Fraggers "fraggers.co.uk")
+    (FreedomChat "freedomchat.net")
+    (FreedomIRC "freedomirc.net")
+    (freenode "freenode.net")
+    (FunNet "funnet.org")
+    (GalaxyNet "galaxynet.org")
+    (Gamesnet "gamesnet.net")
+    (GammaForce "gammaforce.org")
+    (GIKInet "giki.edu.pk")
+    (GizNet "giznet.org")
+    (Globalchat "globalchat.org")
+    (GlobIRC "globirc.net")
+    (Goldchat "goldchat.nl")
+    (Goodchatting "goodchatting.com")
+    (GravityLords "gravitylords.net")
+    (GRnet "irc.gr")
+    (GulfChat "gulfchat.net")
+    (HabberNet "habber.net")
+    (HanIRC "hanirc.org")
+    (Hellenicnet "mirc.gr")
+    (IceNet "icenet.org.za")
+    (ICQnet "icq.com")
+    (iip "anon.iip")
+    (Infatech "infatech.net")
+    (Infinity "infinity-irc.org")
+    (Infomatrix "infomatrix.net")
+    (Inside3D "inside3d.net")
+    (InterlinkChat "interlinkchat.net")
+    (IRC-Chile "irc.cl")
+    (IRC-Hispano "irc-hispano.org")
+    (IRCchat "ircchat.tk")
+    (IRCGate "ircgate.net")
+    (IRCGeeks "ircgeeks.org")
+    (IRChat "irchat.net")
+    (IrcLordz "irclordz.com")
+    (IrcMalta "ircmalta.org")
+    (IRCnet nil)
+    (IRCSoulZ "ircsoulz.net")
+    (IRCSul "wnet.com.br")
+    (IrcTalk "irctalk.net")
+    (Irctoo "irctoo.net")
+    (IRCtown "irc.irctown.net")
+    (IRCworld "ircworld.org")
+    (ircXtreme "ircXtreme.net")
+    (Israelnet "israel.net")
+    (K0wNet "k0w.net")
+    (KDFSnet "kdfs.net")
+    (Kemik "kemik.net")
+    (Kewl\.Org "kewl.org")
+    (Kickchat "kickchat.com")
+    (KidsWorld "kidsworld.org")
+    (Knightnet "knightnet.net")
+    (Konfido\.Net "konfido.net")
+    (Kreynet "krey.net")
+    (Krono "krono.net")
+    (Krushnet "krushnet.org")
+    (LagNet "lagnet.org.za")
+    (Librenet "librenet.net")
+    (LinkNet "link-net.org")
+    (Liquidized "liquidized.net")
+    (M-IRC "m-sys.org")
+    (MagicStar "magicstar.net")
+    (Mavra "mavra.net")
+    (MediaDriven "mediadriven.com")
+    (mIRC-X "mircx.com")
+    (Morat "morat.net")
+    (MusicCity "musiccity.com")
+    (Mysteria "mysteria.net")
+    (Mysterychat "mysterychat.net")
+    (Mystical "mystical.net")
+    (Narancs "narancs.com")
+    (Net-France "net-france.com")
+    (Nevernet "nevernet.net")
+    (Newnet "newnet.net")
+    (Nexusirc "nexusirc.org")
+    (NightStar "nightstar.net")
+    (NitrousNet "nitrousnet.net")
+    (Novernet "novernet.com")
+    (Nullrouted "nullrouted.org")
+    (NullusNet "nullus.net")
+    (OpChat "opchat.org")
+    (Openprojects "openprojects.net")
+    (Othernet "othernet.org")
+    (OtherSide "othersideirc.net")
+    (Outsiderz "outsiderz.com")
+    (OzOrg "oz.org")
+    (Peacefulhaven "peacefulhaven.net")
+    (PhazedIRC "phazedirc.net")
+    (Philchat "philchat.net")
+    (phrozN "phrozn.net")
+    (PiNet "praetorians.org")
+    (Pinoycentral "abs-cbn.com")
+    (Planetarion "planetarion.com")
+    (POLNet "ircnet.pl")
+    (Psionics "psionics.net")
+    (PTirc "ptirc.com.pt")
+    (PTlink "ptlink.net")
+    (PTnet "ptnet.org")
+    (QChat "qchat.net")
+    (QuakeNet "quakenet.org")
+    (Realirc "realirc.org")
+    (RealmNET "realmnet.com")
+    (Rebelchat "rebelchat.org")
+    (Red-Latina "red-latina.org")
+    (RedLatona "redlatona.net")
+    (Relicnet "relic.net")
+    (Rezosup "rezosup.org")
+    (Risanet "risanet.com")
+    (Rubiks "rubiks.net")
+    (Rusnet "nil")
+    (Sandnet "sandnet.net")
+    (Scunc "scunc.net")
+    (SerbianCafe "serbiancafe.ws")
+    (SexNet "sexnet.org")
+    (ShadowFire "shadowfire.org")
+    (ShadowWorld "shadowworld.net")
+    (SkyNet "bronowski.pl")
+    (SlashNET "slashnet.org")
+    (SolarStone "solarstone.net")
+    (Sorcery "sorcery.net")
+    (SourceIRC "sourceirc.net")
+    (SpaceTronix "spacetronix.net")
+    (Spirit-Harmony "spirit-harmony.com")
+    (StarChat "starchat.net")
+    (StarEquinox "starequinox.net")
+    (Starlink "starlink.net")
+    (starlink-irc "starlink-irc.org")
+    (StarWars-IRC "starwars-irc.net")
+    (Stormdancing "stormdancing.net")
+    (Superchat "superchat.org")
+    (Sysopnet "sysopnet.org")
+    (Telstra "telstra.com")
+    (TR-net "dominet.com.tr")
+    (Tri-net "tri-net.org")
+    (TriLink "ft4u.net")
+    (TurkishChat "turkishchat.org")
+    (UberNinja "uberninja.net")
+    (UICN "uicn.net")
+    (UltraIRC "ultrairc.net")
+    (UnderChat "underchat.it")
+    (Undernet "undernet.org")
+    (UnderZ "underz.org")
+    (UniChat "irc.uni-chat.net")
+    (UnionLatina "unionlatina.org")
+    (Univers "univers.org")
+    (UnixR "unixr.net")
+    (Vidgamechat "vidgamechat.com")
+    (VirtuaNet "virtuanet.org")
+    (Vitamina "vitamina.ca")
+    (Voila "voila.fr")
+    (Wahou "wf-net.org")
+    (Warpednet "warped.net")
+    (Weaklinks "weaklinks.net")
+    (Webnet "webchat.org")
+    (WinChat "winchat.net")
+    (WinIRC "winirc.org")
+    (WorldIRC "worldirc.org")
+    (WyldRyde "wyldryde.net")
+    (XentoniX "xentonix.net")
+    (Xevion "xevion.net")
+    (XNet "xnet.org")
+    (XWorld "xworld.org")
+    (ZAnetNet "zanet.net")
+    (ZAnetOrg "zanet.org.za")
+    (ZiRC "zirc.org")
+    (ZUHnet "zuh.net")
+    (Zurna "zurna.net"))
   "Alist of IRC networks. (NET MATCHER) where
 NET is a symbol naming that IRC network and
 MATCHER is used to find a corresponding network to a server while connected to
   it. If it is regexp, it's used to match against `erc-announced-server-name'.
   It can also be a function (predicate). Then it is executed with the
   server buffer as current-buffer."
-  :group 'erc
-  :type 'sexp)
+  :group 'erc-networks
+  :type '(repeat
+          (list :tag "Network"
+                (symbol :tag "Network name")
+		(choice :tag "Network's common server ending"
+		 (regexp)
+		 (const :tag "Network has no common server ending" nil)))))
+
+(defvar erc-network nil
+  "The name of the network you are connected to (a symbol).")
+(make-variable-buffer-local 'erc-network)
+
+;; Functions:
+
+;;;###autoload
+(defun erc-determine-network ()
+  "Return the name of the network or \"Unknown\" as a symbol.  Use the
+server parameter NETWORK if provided, otherwise parse the server name and
+search for a match in `erc-networks-alist'."
+  ;; The server made it easy for us and told us the name of the NETWORK
+  (if (assoc "NETWORK" erc-server-parameters)
+      (intern (cdr (assoc "NETWORK" erc-server-parameters)))
+    (or
+     ;; Loop through `erc-networks-alist' looking for a match.
+     (let ((server (or erc-announced-server-name erc-session-server)))
+       (loop for (name matcher) in erc-networks-alist
+             when (and matcher
+                       (string-match (concat matcher "\\'") server))
+             do (return name)))
+     'Unknown)))
+
+(defun erc-network ()
+  "Return the value of `erc-network' for the current server."
+  (with-current-buffer (erc-server-buffer) erc-network))
+
+(defun erc-current-network ()
+  "Deprecated.  Use `erc-network' instead.  Return the name of this server's
+network as a symbol."
+  (with-current-buffer (erc-server-buffer)
+    (intern (downcase (symbol-name erc-network)))))
+
+(condition-case nil
+    (make-obsolete 'erc-current-network 'erc-network "Obsolete since erc-nets 1.5")
+  (wrong-number-of-arguments (make-obsolete 'erc-current-network 'erc-network)))
+
+(defun erc-network-name ()
+  "Returns the name of the current network as a string."
+  (with-current-buffer (erc-server-buffer) (symbol-name erc-network)))
+
+;; Why was this function marked obsolete?  There is no other function that
+;; returns the name of the current network as a string.  Some of us use these
+;; functions in our personal ERC configuration code.  (franl, 2004-03-13)
+;;
+;;(condition-case nil
+;;    (make-obsolete 'erc-network-name 'erc-network "Obsolete since erc-nets 1.5")
+;;  (wrong-number-of-arguments (make-obsolete 'erc-network-name 'erc-network)))
+
+(defun erc-set-network-name (proc parsed)
+  "Set `erc-network' to the value returned by `erc-determine-network'."
+  (unless erc-connected
+    (setq erc-network (erc-determine-network)))
+  nil)
+
+(defun erc-unset-network-name (nick ip reason)
+  "Set `erc-network' to nil."
+  (setq erc-network nil)
+  nil)
+
+(add-hook 'erc-server-375-functions 'erc-set-network-name)
+(add-hook 'erc-server-422-functions 'erc-set-network-name)
+(add-hook 'erc-disconnected-hook 'erc-unset-network-name)
 
 (defun erc-ports-list (ports)
-  (let (plst)
+  "Return a list of PORTS.
+
+PORTS should be a list of either:
+  A number, in which case it is returned a list.
+  Or a pair of the form (LOW HIGH), in which case, a list of all the
+  numbers between LOW and HIGH (inclusive) is returned.
+
+As an example:
+  (erc-ports-list '(1)) => (1)
+  (erc-ports-list '((1 5))) => (1 2 3 4 5)
+  (erc-ports-list '(1 (3 5))) => (1 3 4 5)"
+  (let (result)
     (dolist (p ports)
       (cond ((numberp p)
-	     (setq plst (append plst (list p))))
+	     (push p result))
 	    ((listp p)
-	     (setq plst
-		   (append plst
-			   (loop for i from (car p) to (nth 1 p) collect i))))))
-    plst))
+	     (setq result (nconc (loop for i from (cadr p) downto (car p)
+                                       collect i)
+                                 result)))))
+    (nreverse result)))
 
 ;;;###autoload
 (defun erc-server-select ()
-  "Interactively select a server using the `erc-networks-alist' and
-`erc-server-alist' to connect to."
+  "Interactively select a server to connect to using `erc-server-alist'."
   (interactive)
   (let* ((completion-ignore-case t)
 	 (net (intern
 	       (completing-read "Network: "
-				(mapcar (lambda (x)
-					  (list (symbol-name (car x))))
-					erc-networks-alist))))
+				(remove-duplicates
+				 (mapcar (lambda (x)
+					   (list (symbol-name (nth 1 x))))
+					 erc-server-alist)
+				 :test #'equal))))
 	 (srv (assoc
 	       (completing-read "Server: "
 				(delq nil
@@ -693,17 +810,16 @@ MATCHER is used to find a corresponding network to a server while connected to
 ;; It does not work yet, help me with it if you
 ;; think it is worth the effort.
 
-(defcustom erc-settings 
-  '((pals Openprojects ("kensanata" "shapr" "anti\\(fuchs\\|gone\\)"))
-    (format-nick-function (Openprojects "#emacs") erc-format-@nick))
-  "Alist of configuration options. (VARNAME SCOPE VALUE) where
+(defvar erc-settings 
+  '((pals freenode ("kensanata" "shapr" "anti\\(fuchs\\|gone\\)"))
+    (format-nick-function (freenode "#emacs") erc-format-@nick))
+  "Experimental: Alist of configuration options.
+The format is (VARNAME SCOPE VALUE) where
 VARNAME is a symbol identifying the configuration option,
 SCOPE is either a symbol which identifies an entry from
   `erc-networks-alist' or a list (NET TARGET) where NET is a network symbol and
   TARGET is a string identifying the channel/query target.
-VALUE is the options value."
-  :group 'erc
-  :type 'sexp)
+VALUE is the options value.")
 
 (defun erc-get (var &optional net target)
   (let ((items erc-settings)
@@ -725,9 +841,17 @@ VALUE is the options value."
 		     items nil)))))
     val))
 
-(erc-get 'pals 'Openprojects)
+(erc-get 'pals 'freenode)
 	       
 	       
   
 
 (provide 'erc-nets)
+
+;;; erc-nets.el ends here
+;;
+;; Local Variables:
+;; indent-tabs-mode: t
+;; tab-width: 8
+;; standard-indent: 4
+;; End:
