@@ -1,6 +1,6 @@
 ;;; erc-track.el --- Track modified channel buffers
 
-;; Copyright (C) 2002  Mario Lang <mlang@delysid.org>
+;; Copyright (C) 2002,2003,2004,2005 Free Software Foundation, Inc.
 
 ;; Author: Mario Lang <mlang@delysid.org>
 ;; Keywords: comm, faces
@@ -23,7 +23,7 @@
 
 ;;; Commentary:
 
-;; Highlights keywords and pals (friends), and hides or hilights fools
+;; Highlights keywords and pals (friends), and hides or highlights fools
 ;; (using a dark color).  Add to your ~/.emacs:
 
 ;; (require 'erc-track)
@@ -39,11 +39,12 @@
 
 ;;; Code:
 
-(defconst erc-track-version "$Revision: 1.58 $"
+(defconst erc-track-version "$Revision: 1.72.2.2 $"
   "ERC track mode revision")
 
 (defgroup erc-track nil
-  "Track active buffers and show activity in the modeline.")
+  "Track active buffers and show activity in the modeline."
+  :group 'erc)
 
 (defcustom erc-track-visibility t
   "Where do we look for buffers to determine their visibility?
@@ -77,6 +78,13 @@ This list could look like '(\"JOIN\" \"PART\")."
   :group 'erc-track
   :type 'erc-message-type)
 
+(defcustom erc-track-exclude-server-buffer nil
+  "*If true, don't perform tracking on the server buffer; this is
+useful for excluding all the things like MOTDs from the server and
+other miscellaneous functions."
+  :group 'erc-track
+  :type 'boolean)
+
 (defcustom erc-track-shorten-start 1
   "This number specifies the minimum number of characters a channel name in
 the mode-line should be reduced to."
@@ -139,10 +147,63 @@ be highlighted using that face.  The first matching face is used."
   :group 'erc-track
   :type '(repeat face))
 
-(defvar erc-modified-channels-string ""
-  "Internal string used for displaying modified channels in the mode line.")
+(defcustom erc-track-priority-faces-only nil
+  "If you would like to ignore changes in certain channels where there
+are no faces corresponding to your erc-track-faces-priority-list, set
+this variable.  You can set a list of channel name strings, so those
+will be ignored while all other channels will be tracked as normal.
+Other options are 'all, to apply this to all channels or nil, to disable
+this feature.
+Note: If you have a lot of faces listed in erc-track-faces-priority-list,
+setting this variable might not be very useful."
+  :group 'erc-track
+  :type '(choice (const nil)
+		 (repeat string)
+		 (const all)))
 
-(put 'erc-modified-channels-string 'risky-local-variable t); allow properties
+(defcustom erc-track-position-in-mode-line 'before-modes
+  "Where to show modified channel information in the mode-line.
+
+Setting this variable only has effects in GNU Emacs versions above 21.3.
+
+Choices are:
+'before-modes - add to the beginning of `mode-line-modes'
+'after-modes  - add to the end of `mode-line-modes'
+
+Any other value means add to the end of `global-mode-string'."
+  :group 'erc-track
+  :type '(choice (const :tag "Just before mode information" before-modes)
+                 (const :tag "Just after mode information" after-modes)
+                 (const :tag "After all other information" nil))
+  :set (lambda (sym val)
+         (set sym val)
+         (when (and (boundp 'erc-track-mode)
+                    erc-track-mode)
+           (erc-track-remove-from-mode-line)
+           (erc-track-add-to-mode-line val))))
+
+(defun erc-modified-channels-object (strings)
+  "Generate a new `erc-modified-channels-object' based on STRINGS.
+If STRINGS is nil, we initialize `erc-modified-channels-object' to
+an appropriate initial value for this flavor of Emacs."
+  (if strings
+      (if (featurep 'xemacs)
+          (let ((e-m-c-s '("[")))
+            (push (cons (extent-at 0 (car strings)) (car strings))
+                  e-m-c-s)
+            (dolist (string (cdr strings))
+              (push "," e-m-c-s)
+              (push (cons (extent-at 0 string) string)
+                    e-m-c-s))
+            (push "] " e-m-c-s)
+            (reverse e-m-c-s))
+        (concat " [" (mapconcat 'identity (nreverse strings) ",") "] "))
+    (if (featurep 'xemacs) '() "")))
+
+(defvar erc-modified-channels-object (erc-modified-channels-object nil)
+  "Internal object used for displaying modified channels in the mode line.")
+
+(put 'erc-modified-channels-object 'risky-local-variable t); allow properties
 
 (defvar erc-modified-channels-alist nil
   "An ALIST used for tracking channel modification activity.
@@ -152,9 +213,38 @@ indicating how often activity was noticed, and FACE is the face to use
 when displaying the buffer's name.  See `erc-track-faces-priority-list',
 and `erc-track-showcount'.
 
-Entries in this list should only happen for buffers where activity occured
+Entries in this list should only happen for buffers where activity occurred
 while the buffer was not visible.")
 
+
+(defun erc-track-remove-from-mode-line ()
+  "Remove `erc-track-modified-channels' from the mode-line"
+  (when (boundp 'mode-line-modes)
+    (setq mode-line-modes
+          (remove '(t erc-modified-channels-object) mode-line-modes)))
+  (setq global-mode-string
+        (delq 'erc-modified-channels-object global-mode-string)))
+
+(defun erc-track-add-to-mode-line (position)
+  "Add `erc-track-modified-channels' to POSITION in the mode-line.
+See `erc-track-position-in-mode-line' for possible values."
+  ;; CVS Emacs has a new format string, and global-mode-string
+  ;; is very far to the right.
+  (cond ((and (eq position 'before-modes)
+              (boundp 'mode-line-modes))
+         (add-to-list 'mode-line-modes
+                      '(t erc-modified-channels-object)))
+        ((and (eq position 'after-modes)
+              (boundp 'mode-line-modes))
+         (add-to-list 'mode-line-modes
+                      '(t erc-modified-channels-object) t))
+        (t
+         (when (not global-mode-string)
+           (setq global-mode-string '(""))) ; Padding for mode-line wart
+         (add-to-list 'global-mode-string
+                      'erc-modified-channels-object
+                      t))))
+
 ;;; Shortening of names
 
 (defun erc-track-shorten-names (channel-names)
@@ -242,7 +332,7 @@ START is the minimum length of the name used."
 	   ;; do not shorten if a predicate exists and it returns nil
 	   str
 	 ;; Start with smallest substring candidate, ie. length 1.
-	 ;; Then check all the others and see wether any of them starts
+	 ;; Then check all the others and see whether any of them starts
 	 ;; with the same substring.  While there is such another
 	 ;; element in the list, increase the length of the candidate.
 	 (while (not done)
@@ -275,113 +365,107 @@ START is the minimum length of the name used."
  (and
   ;; verify examples from the doc strings
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("#emacs" "#vi" "#electronica" "#folk")
-	    '("#emacs" "#vi")))
-	 '("#em" "#vi")); emacs is different from electronica
+           (erc-unique-channel-names
+            '("#emacs" "#vi" "#electronica" "#folk")
+            '("#emacs" "#vi")))
+         '("#em" "#vi"))	 ; emacs is different from electronica
   (equal (let ((erc-track-shorten-aggressively t))
-	   (erc-unique-channel-names
-	    '("#emacs" "#vi" "#electronica" "#folk")
-	    '("#emacs" "#vi")))
-	 '("#em" "#v")); vi is shortened by one letter
+           (erc-unique-channel-names
+            '("#emacs" "#vi" "#electronica" "#folk")
+            '("#emacs" "#vi")))
+         '("#em" "#v"))                ; vi is shortened by one letter
   (equal (let ((erc-track-shorten-aggressively 'max))
-	   (erc-unique-channel-names
-	    '("#emacs" "#vi" "#electronica" "#folk")
-	    '("#emacs" "#vi")))
-	 '("#e" "#v")); emacs need not be different from electronica
+           (erc-unique-channel-names
+            '("#emacs" "#vi" "#electronica" "#folk")
+            '("#emacs" "#vi")))
+         '("#e" "#v"))  ; emacs need not be different from electronica
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("#linux-de" "#linux-fr")
-	    '("#linux-de" "#linux-fr")))
-	 '("#linux-de" "#linux-fr")); shortening by one letter is too aggressive
+           (erc-unique-channel-names
+            '("#linux-de" "#linux-fr")
+            '("#linux-de" "#linux-fr")))
+         '("#linux-de" "#linux-fr")) ; shortening by one letter is too aggressive
   (equal (let ((erc-track-shorten-aggressively t))
 	   (erc-unique-channel-names
 	    '("#linux-de" "#linux-fr")
 	    '("#linux-de" "#linux-fr")))
-	 '("#linux-d" "#linux-f")); now we want to be agressive
+	 '("#linux-d" "#linux-f")); now we want to be aggressive
   ;; specific problems
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("#dunnet" "#lisp" "#sawfish" "#fsf" "#guile"
-	      "#testgnome" "#gnu" "#fsbot" "#hurd" "#hurd-bunny"
-	      "#emacs")
-	    '("#hurd-bunny" "#hurd" "#sawfish" "#lisp")))
-	   '("#hurd-" "#hurd" "#s" "#l"))
+           (erc-unique-channel-names
+            '("#dunnet" "#lisp" "#sawfish" "#fsf" "#guile"
+              "#testgnome" "#gnu" "#fsbot" "#hurd" "#hurd-bunny"
+              "#emacs")
+            '("#hurd-bunny" "#hurd" "#sawfish" "#lisp")))
+         '("#hurd-" "#hurd" "#s" "#l"))
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-substrings
-	    '("#emacs" "#vi" "#electronica" "#folk")))
-	 '("#em" "#vi" "#el" "#f"))
+           (erc-unique-substrings
+            '("#emacs" "#vi" "#electronica" "#folk")))
+         '("#em" "#vi" "#el" "#f"))
   (equal (let ((erc-track-shorten-aggressively t))
-	   (erc-unique-substrings
-	    '("#emacs" "#vi" "#electronica" "#folk")))
-	 '("#em" "#v" "#el" "#f"))
+           (erc-unique-substrings
+            '("#emacs" "#vi" "#electronica" "#folk")))
+         '("#em" "#v" "#el" "#f"))
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("#emacs" "#burse" "+linux.de" "#starwars"
-	      "#bitlbee" "+burse" "#ratpoison")
-	    '("+linux.de" "#starwars" "#burse")))
-	 '("+l" "#s" "#bu"))
+           (erc-unique-channel-names
+            '("#emacs" "#burse" "+linux.de" "#starwars"
+              "#bitlbee" "+burse" "#ratpoison")
+            '("+linux.de" "#starwars" "#burse")))
+         '("+l" "#s" "#bu"))
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("fsbot" "#emacs" "deego")
-	    '("fsbot")))
-	 '("fs"))
+           (erc-unique-channel-names
+            '("fsbot" "#emacs" "deego")
+            '("fsbot")))
+         '("fs"))
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("fsbot" "#emacs" "deego")
-	    '("fsbot")
-	    (lambda (s)
-	      (> (length s) 4))
-	    1))
-	 '("f"))
+           (erc-unique-channel-names
+            '("fsbot" "#emacs" "deego")
+            '("fsbot")
+            (lambda (s)
+              (> (length s) 4))
+            1))
+         '("f"))
   (equal (let ((erc-track-shorten-aggressively nil))
-	   (erc-unique-channel-names
-	    '("fsbot" "#emacs" "deego")
-	    '("fsbot")
-	    (lambda (s)
-	      (> (length s) 4))
-	    2))
-	 '("fs"))
+           (erc-unique-channel-names
+            '("fsbot" "#emacs" "deego")
+            '("fsbot")
+            (lambda (s)
+              (> (length s) 4))
+            2))
+         '("fs"))
   (let ((erc-track-shorten-aggressively nil))
     (equal (erc-unique-channel-names '("deego" "#hurd" "#hurd-bunny" "#emacs")
-				     '("#hurd" "#hurd-bunny"))
-	   '("#hurd" "#hurd-")))
+                                     '("#hurd" "#hurd-bunny"))
+           '("#hurd" "#hurd-")))
   ;; general examples
   (let ((erc-track-shorten-aggressively t))
     (and (equal (erc-unique-substring-1 "abc" '("ab" "abcd")) "abcd")
-	 (not (erc-unique-substring-1 "a" '("xyz" "xab")))
-	 (equal (erc-unique-substrings '("abc" "xyz" "xab")) 
-		'("ab" "xy" "xa"))
-	 (equal (erc-unique-substrings '("abc" "abcdefg"))
-		'("abc" "abcd"))))
+         (not (erc-unique-substring-1 "a" '("xyz" "xab")))
+         (equal (erc-unique-substrings '("abc" "xyz" "xab")) 
+                '("ab" "xy" "xa"))
+         (equal (erc-unique-substrings '("abc" "abcdefg"))
+                '("abc" "abcd"))))
   (let ((erc-track-shorten-aggressively nil))
     (and (equal (erc-unique-substring-1 "abc" '("ab" "abcd")) "abcd")
-	 (not (erc-unique-substring-1 "a" '("xyz" "xab")))
-	 (equal (erc-unique-substrings '("abc" "xyz" "xab")) 
-		'("abc" "xyz" "xab"))
-	 (equal (erc-unique-substrings '("abc" "abcdefg")) 
-		'("abc" "abcd"))))))
+         (not (erc-unique-substring-1 "a" '("xyz" "xab")))
+         (equal (erc-unique-substrings '("abc" "xyz" "xab")) 
+                '("abc" "xyz" "xab"))
+         (equal (erc-unique-substrings '("abc" "abcdefg")) 
+                '("abc" "abcd"))))))
 
 ;;; Module
 
 ;;;###autoload (autoload 'erc-track-mode "erc-track" nil t)
 (define-erc-module track track-modified-channels
   "This mode tracks ERC channel buffers with activity."
-  ((or global-mode-string
-       (setq global-mode-string '("")))
-   (or (memq 'erc-modified-channels-string global-mode-string)
-       (setq global-mode-string
-	     (append global-mode-string
-		     '(erc-modified-channels-string))))
-   (setq erc-modified-channels-string "")
+  ((erc-track-add-to-mode-line erc-track-position-in-mode-line)
+   (setq erc-modified-channels-object (erc-modified-channels-object nil))
    (erc-update-mode-line)
    (if (featurep 'xemacs)
        (defadvice switch-to-buffer (after erc-update (&rest args) activate)
          (erc-modified-channels-update))
      (add-hook 'window-configuration-change-hook 'erc-modified-channels-update))
    (add-hook 'erc-insert-post-hook 'erc-track-modified-channels))
-  ((setq global-mode-string
-	 (delq 'erc-modified-channels-string global-mode-string))
+  ((erc-track-remove-from-mode-line)
    (if (featurep 'xemacs)
        (ad-disable-advice 'switch-to-buffer 'after 'erc-update)
      (remove-hook 'window-configuration-change-hook 'erc-modified-channels-update))
@@ -396,14 +480,13 @@ if you are inactivity."
          (erc-user-is-active))
      (add-hook 'window-configuration-change-hook 'erc-user-is-active))
    (add-hook 'erc-send-completed-hook 'erc-user-is-active)
-   (add-hook 'erc-server-001-hook 'erc-user-is-active))
-  ((setq global-mode-string
-	 (delq 'erc-modified-channels-string global-mode-string))
+   (add-hook 'erc-server-001-functions 'erc-user-is-active))
+  ((erc-track-remove-from-mode-line)
    (if (featurep 'xemacs)
        (ad-disable-advice 'switch-to-buffer 'after 'erc-update-when-inactive)
      (remove-hook 'window-configuration-change-hook 'erc-user-is-active))
    (remove-hook 'erc-send-completed-hook 'erc-user-is-active)
-   (remove-hook 'erc-server-001-hook 'erc-user-is-active)
+   (remove-hook 'erc-server-001-functions 'erc-user-is-active)
    (remove-hook 'erc-timer-hook 'erc-user-is-active)))
 
 ;;; Visibility
@@ -489,7 +572,7 @@ If FACES are provided, color STRING with them."
     name))
 
 (defun erc-modified-channels-display ()
-  "Set `erc-modified-channels-string'
+  "Set `erc-modified-channels-object'
 according to `erc-modified-channels-alist'.
 Use `erc-make-mode-line-buffer-name' to create buttons."
   (if (or
@@ -497,12 +580,12 @@ Use `erc-make-mode-line-buffer-name' to create buttons."
 	(eq 'leastactive erc-track-switch-direction))
       (erc-track-sort-by-activest))
   (if (null erc-modified-channels-alist)
-      (setq erc-modified-channels-string "")
+      (setq erc-modified-channels-object (erc-modified-channels-object nil))
     ;; erc-modified-channels-alist contains all the data we need.  To
     ;; better understand what is going on, we split things up into
     ;; four lists: BUFFERS, COUNTS, SHORT-NAMES, and FACES.  These
     ;; four lists we use to create a new
-    ;; `erc-modified-channels-string' using
+    ;; `erc-modified-channels-object' using
     ;; `erc-make-mode-line-buffer-name'.
     (let* ((buffers (mapcar 'car erc-modified-channels-alist))
 	   (counts (mapcar 'cadr erc-modified-channels-alist))
@@ -524,8 +607,10 @@ Use `erc-make-mode-line-buffer-name' to create buttons."
 	      buffers (cdr buffers)
 	      counts (cdr counts)
 	      faces (cdr faces)))
-      (setq erc-modified-channels-string
-	    (concat "[" (mapconcat 'identity (nreverse strings) ",") "] ")))))
+      (when (featurep 'xemacs)
+        (erc-modified-channels-object nil))
+      (setq erc-modified-channels-object
+            (erc-modified-channels-object strings)))))
 
 (defun erc-modified-channels-remove-buffer (buffer)
   "Remove BUFFER from `erc-modified-channels-alist'."
@@ -554,40 +639,52 @@ first in that list will be used."
 buffer should be added to the modeline as a hidden, modified
 channel.  Assumes it will only be called when current-buffer
 is in `erc-mode'."
-  (let ((this-channel (erc-default-target)))
+  (let ((this-channel (or (erc-default-target) (buffer-name (current-buffer)))))
     (if (and (not (erc-buffer-visible (current-buffer)))
 	     (not (member this-channel erc-track-exclude))
+ 	     (not (and erc-track-exclude-server-buffer
+                       (string= this-channel
+                                (buffer-name (erc-server-buffer)))))
 	     (not (erc-message-type-member
 		   (or (erc-find-parsed-property)
                        (point-min))
 		   erc-track-exclude-types)))
 	;; If the active buffer is not visible (not shown in a
 	;; window), and not to be excluded, determine the kinds of
-	;; faces used in the current message, and add the buffer to
-	;; the erc-modified-channels-alist, if it is not already
-	;; there.  If it the buffer is already on the list (in the
-	;; car), change its face attribute (in the cddr) if necessary.
-        ;; See `erc-modified-channels-alist' for the exact data
-        ;; structure used.
+	;; faces used in the current message, and unless the user
+	;; wants to ignore changes in certain channels where there
+	;; are no faces corresponding to `erc-track-faces-priority-list',
+	;; and the faces in the current message are found in said
+	;; priority list, add the buffer to the erc-modified-channels-alist,
+	;; if it is not already there.  If the buffer is already on the list
+	;; (in the car), change its face attribute (in the cddr) if
+	;; necessary.  See `erc-modified-channels-alist' for the
+	;; exact data structure used.
 	(let ((faces (erc-faces-in (buffer-string))))
-	  (if (not (assq (current-buffer) erc-modified-channels-alist))
-	      ;; Add buffer, faces and counts
-	      (setq erc-modified-channels-alist
-		    (cons (cons (current-buffer)
-				(cons 1 (erc-track-find-face faces)))
-			  erc-modified-channels-alist))
-	    ;; Else modify the face for the buffer, if necessary.
-	    (when faces
- 	      (let* ((cell (assq (current-buffer)
-				erc-modified-channels-alist))
-		     (old-face (cddr cell))
-		     (new-face (erc-track-find-face
-				(if old-face
-				    (cons old-face faces)
-				  faces))))
-		(setcdr cell (cons (1+ (second cell)) new-face)))))
-	  ;; And display it
-	  (erc-modified-channels-display))
+	  (unless (and
+		   (or (eq erc-track-priority-faces-only 'all)
+		       (member this-channel erc-track-priority-faces-only))
+		   (not (dolist (f faces)
+			  (when (member f erc-track-faces-priority-list)
+			    (return t)))))
+	    (if (not (assq (current-buffer) erc-modified-channels-alist))
+		;; Add buffer, faces and counts
+		(setq erc-modified-channels-alist
+		      (cons (cons (current-buffer)
+				  (cons 1 (erc-track-find-face faces)))
+			    erc-modified-channels-alist))
+	      ;; Else modify the face for the buffer, if necessary.
+	      (when faces
+		(let* ((cell (assq (current-buffer)
+				   erc-modified-channels-alist))
+		       (old-face (cddr cell))
+		       (new-face (erc-track-find-face
+				  (if old-face
+				      (cons old-face faces)
+				    faces))))
+		  (setcdr cell (cons (1+ (second cell)) new-face)))))
+	    ;; And display it
+	    (erc-modified-channels-display)))
       ;; Else if the active buffer is the current buffer, remove it
       ;; from our list.
       (when (or (erc-buffer-visible (current-buffer))
@@ -613,12 +710,12 @@ is in `erc-mode'."
 (assert
  (let ((str "is bold"))
    (put-text-property 3 (length str)
-		      'face '(bold erc-current-nick-face)
-		      str)
+                      'face '(bold erc-current-nick-face)
+                      str)
    (erc-faces-in str)))
 
 (defun erc-find-parsed-property ()
-  "Find the next occurence of the `erc-parsed' text property."
+  "Find the next occurrence of the `erc-parsed' text property."
   (let ((pos (point-min)))
     (while (and pos (not (get-text-property pos 'erc-parsed)))
       (setq pos (next-single-property-change pos 'erc-parsed)))
@@ -647,7 +744,7 @@ The default is a colon, resulting in \"#emacs:9\"."
   "Direction `erc-track-switch-buffer' should switch.
 'oldest will find the oldest active buffer.
 'newest finds the latest, 'leastactive finds buffer with least unseen messages,
-'mostactive - with most unseeen messages."
+'mostactive - with most unseen messages."
   :group 'erc-track
   :type '(choice (const oldest) (const newest) (const leastactive) (const mostactive)))
 
